/*
 * Copyright 2017 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.kappich.pat.gnd.
 * 
 * de.kappich.pat.gnd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.kappich.pat.gnd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.gnd.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.kappich.pat.gnd.areaPlugin;

import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.kappich.pat.gnd.displayObjectToolkit.*;
import de.kappich.pat.gnd.gnd.MapPane;
import de.kappich.pat.gnd.layerManagement.Layer;
import de.kappich.pat.gnd.pluginInterfaces.DisplayObjectPainter;
import de.kappich.pat.gnd.pluginInterfaces.DisplayObjectType;
import de.kappich.pat.gnd.pluginInterfaces.DisplayObjectTypePlugin;
import de.kappich.pat.gnd.pluginInterfaces.DisplayObjectsInitializer;

import javax.swing.JProgressBar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Dies ist die Implementierung des <code>DisplayObjectsInitializers</code> im Area-Plugin.
 *
 * @author Kappich Systemberatung
 * @version $Revision$
 */
public class AreaInitializer implements DisplayObjectsInitializer {

	private static final Debug _debug = Debug.getLogger();

	@Override
	public void initializeDisplayObjects(final DataModel configuration, final Layer layer,
			final MapPane mapPane, final JProgressBar progressBar, List<DisplayObject> returnList) {
		final String geoReferenceType = layer.getConfigurationObjectType();
		final SystemObjectType systemObjectType = configuration.getType(geoReferenceType);
		if(systemObjectType==null) {
			_debug.warning("Der Typ " + geoReferenceType + " ist in der Konfiguration unbekannt.");
			return;
		}
		SystemObjectType areaType = configuration.getType("typ.fläche");
		if(!systemObjectType.inheritsFrom(areaType)) {
			_debug.warning("Der Typ " + areaType.getName() + " ist kein Flächentyp.");
			return;
		}

		final DOTCollection dotCollection = layer.getDotCollection();
		final Iterator<DisplayObjectType> iterator = dotCollection.values().iterator();
		if(!iterator.hasNext()) {
			return;
		}
		final DisplayObjectType dot = iterator.next();
		final DisplayObjectTypePlugin displayObjectTypePlugin = dot.getDisplayObjectTypePlugin();
		DisplayObjectPainter painter = displayObjectTypePlugin.getPainter();
		final Map<DisplayObjectType, List<PrimitiveFormPropertyPair>> pfpPairs = dotCollection.getPrimitiveFormPropertyPairs();
		final List<SystemObject> systemObjects = systemObjectType.getElements();
		for(SystemObject systemObject : systemObjects) {
			final List<Object> complexCoordinates = GeoInitializer.getInstance().getAreaCoordinates(systemObject);
			OnlineDisplayObject displayObject = new OnlineDisplayObject(systemObject, complexCoordinates, painter,
			                                                            dotCollection, pfpPairs, mapPane);
			returnList.add(displayObject);
		}
	}
}
